/**
 * The MySensors Arduino library handles the wireless radio link and protocol
 * between your home built sensors/actuators and HA controller of choice.
 * The sensors forms a self healing radio network with optional repeaters. Each
 * repeater and gateway builds a routing tables in EEPROM which keeps track of the
 * network topology allowing messages to be routed to nodes.
 *
 * Created by Henrik Ekblad <henrik.ekblad@mysensors.org>
 * Copyright (C) 2013-2015 Sensnology AB
 * Full contributor list: https://github.com/mysensors/Arduino/graphs/contributors
 *
 * Documentation: http://www.mysensors.org
 * Support Forum: http://forum.mysensors.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 *******************************
 *
 * REVISION HISTORY
 * Version 1.0 - Henrik EKblad
 * Contribution by a-lurker and Anticimex,
 * Contribution by Norbert Truchsess <norbert.truchsess@t-online.de>
 * Contribution by Ivo Pullens (ESP8266 support)
 *
 * DESCRIPTION
 * The EthernetGateway sends data received from sensors to the WiFi link.
 * The gateway also accepts input on ethernet interface, which is then sent out to the radio network.
 *
 * VERA CONFIGURATION:
 * Enter "ip-number:port" in the ip-field of the Arduino GW device. This will temporarily override any serial configuration for the Vera plugin.
 * E.g. If you want to use the defualt values in this sketch enter: 192.168.178.66:5003
 *
 * LED purposes:
 * - To use the feature, uncomment any of the MY_DEFAULT_xx_LED_PINs in your sketch, only the LEDs that is defined is used.
 * - RX (green) - blink fast on radio message recieved. In inclusion mode will blink fast only on presentation recieved
 * - TX (yellow) - blink fast on radio message transmitted. In inclusion mode will blink slowly
 * - ERR (red) - fast blink on error during transmission error or recieve crc error
 *
 * See http://www.mysensors.org/build/esp8266_gateway for wiring instructions.
 * nRF24L01+  ESP8266
 * VCC        VCC
 * CE         GPIO4
 * CSN/CS     GPIO15
 * SCK        GPIO14
 * MISO       GPIO12
 * MOSI       GPIO13
 * GND        GND
 *
 * Not all ESP8266 modules have all pins available on their external interface.
 * This code has been tested on an ESP-12 module.
 * The ESP8266 requires a certain pin configuration to download code, and another one to run code:
 * - Connect REST (reset) via 10K pullup resistor to VCC, and via switch to GND ('reset switch')
 * - Connect GPIO15 via 10K pulldown resistor to GND
 * - Connect CH_PD via 10K resistor to VCC
 * - Connect GPIO2 via 10K resistor to VCC
 * - Connect GPIO0 via 10K resistor to VCC, and via switch to GND ('bootload switch')
 *
  * Inclusion mode button:
 * - Connect GPIO5 via switch to GND ('inclusion switch')
 *
 * Hardware SHA204 signing is currently not supported!
 *
 * Make sure to fill in your ssid and WiFi password below for ssid & pass.
 */

// Enable debug prints to serial monitor
//#define MY_DEBUG

#define DEBUG_m

// Use a bit lower baudrate for serial prints on ESP8266 than default in MyConfig.h
#define MY_BAUD_RATE 9600

// Enables and select radio type (if attached)
#define MY_RADIO_NRF24
//#define MY_RADIO_RFM69
//#define MY_RADIO_RFM95

#define MY_GATEWAY_ESP8266

#define MY_RF24_CE_PIN (2) // CE D4 Освобождаем ногу для I2C протокола

#define MY_RF24_CHANNEL (100)

#define MY_ESP8266_SSID "*****"
#define MY_ESP8266_PASSWORD "******"

// Enable UDP communication
//#define MY_USE_UDP  // If using UDP you need to set MY_CONTROLLER_IP_ADDRESS below

// Set the hostname for the WiFi Client. This is the hostname
// it will pass to the DHCP server if not static.
//#define MY_ESP8266_HOSTNAME "sensor-gateway"

// Enable MY_IP_ADDRESS here if you want a static ip address (no DHCP)
#define MY_IP_ADDRESS 192,168,2,121

// If using static ip you can define Gateway and Subnet address as well
#define MY_IP_GATEWAY_ADDRESS 192,168,2,100
#define MY_IP_SUBNET_ADDRESS 255,255,255,0

// The port to keep open on node server mode
#define MY_PORT 5003

// How many clients should be able to connect to this gateway (default 1)
#define MY_GATEWAY_MAX_CLIENTS 5

// Controller ip address. Enables client mode (default is "server" mode).
// Also enable this if MY_USE_UDP is used and you want sensor data sent somewhere.
//#define MY_CONTROLLER_IP_ADDRESS 192, 168, 178, 68

// Enable inclusion mode
//#define MY_INCLUSION_MODE_FEATURE

// Enable Inclusion mode button on gateway
//#define MY_INCLUSION_BUTTON_FEATURE
// Set inclusion mode duration (in seconds)
//#define MY_INCLUSION_MODE_DURATION 60
// Digital pin used for inclusion mode button
//#define MY_INCLUSION_MODE_BUTTON_PIN  3

// Set blinking period
#define MY_DEFAULT_LED_BLINK_PERIOD 300

// Flash leds on rx/tx/err
// Led pins used if blinking feature is enabled above
#define MY_DEFAULT_ERR_LED_PIN 16  // Error led pin
#define MY_DEFAULT_RX_LED_PIN  16  // Receive led pin
#define MY_DEFAULT_TX_LED_PIN  16  // the PCB, on board LED

#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27,20,4);//set the LCD address to 0x27 for a 16 chars and 2 line display


// время апдейта данных с ноды

  unsigned long lpTempOldUpdate  = 0;
  unsigned long lpTempUpdate  = 0;
  int lpTempUp  = 0;

  unsigned long lpContOldUpdate  = 0;
  unsigned long lpContUpdate  = 0; 
  int lpContUp  = 0;

byte gateAddr[]= {192, 168, 2, 122};    // адрес гейта , с которого читаем данные
int gatePort = 5003;                    // порт по умолчанию


   int data1;                // температура 133 ноды
   int data2;                // влажность   133

   int data3;                // температура 60
   int data4;                // влажность   60
   int data5;                // давление    60


#include <DNSServer.h>
#include <FS.h>


#if defined(MY_USE_UDP)
#include <WiFiUdp.h>
#endif 

 
#include <ESP8266WiFi.h>

  WiFiClient gate;

String dataString = ""; 


#include <MySensors.h>

void setup()
{
	// Setup locally attached sensors

   statusGate();

 delay(2000);

  // Setup locally attached sensors

  lcd.begin();// initialize the lcd with SDA and SCL pins
  
  lcd.backlight();

  // Print a message to the LCD.

  lcd.setCursor(0,0);
  lcd.print("MySens");

  lcd.setCursor(0,1);
  lcd.print("Temp");  

  lcd.setCursor(0,2);
  lcd.print("Humi");

  lcd.setCursor(0,3);
  lcd.print("Pres");  
  
}

void presentation()
{
  //Send the sensor node sketch version information to the gateway
  
  sendSketchInfo("GateWay Node_LCD", "3.2");

}

void loop()
{
	// Send locally attached sensors data here

    reqGate();

        lpTempUpdate = (millis() - lpTempOldUpdate) / 1000; // время последних данных в секундах
        lpTempUp = lpTempUpdate / 60;                       // время последних данных в минутах
        
        lpContUpdate = (millis() - lpContOldUpdate) / 1000;
        lpContUp = lpContUpdate / 60;

        lcd.setCursor(15, 0 );
        lcd.print("   ");                                    //  очищаем экран от незначащих нулей. В кавычках 3 пробела
       
        lcd.setCursor(15, 0 );                               // выводит время последнего обновления датчика
        lcd.print(lpTempUp);

        lcd.setCursor(10, 0 );
        lcd.print("   ");
        
        lcd.setCursor(10, 0 );
        lcd.print(lpContUp);

  
        lcd.setCursor(10, 1);
        lcd.print(data3);

        lcd.setCursor(10, 2);
        lcd.print(data4);       

        lcd.setCursor(10, 3);
        lcd.print(data5);

        lcd.setCursor(15, 2);
        lcd.print(data2);       

        lcd.setCursor(15, 1);
        lcd.print(data1); 

     delay(300);  
     
}

void receive(const MyMessage &message)
{

#ifdef DEBUG_m

    MY_SERIALDEVICE.print(message.sender);

    MY_SERIALDEVICE.print(":");   
    
    MY_SERIALDEVICE.print(message.sensor);
  
    MY_SERIALDEVICE.print(":");

    MY_SERIALDEVICE.println(message.getFloat());
 #endif // DEBUG_m
 
}


void statusGate() {


  Serial.println("***Gate status***");
  if (gate.connect(gateAddr, gatePort)) {

 // if (cl.connect(gateAddr, gatePort)) {  
    Serial.println("Gate connect: Ok");

 // lcd.setCursor(0,0);
 // lcd.print("MySens");
  
  } else {
    Serial.println("Gate connect: Fail");
  } // else 

}   // statusGate()

void reqGate() {                                    // функция чтения данных из порта

  String dataString = "";                           // объявляем dataString как локальную переменную
  
  while(gate.available() > 0) {                     // цикл чтения данных, если есть данные то записываем
    char data = gate.read();                        // запись пришедшего байта

 // MYSENS_DEBUG
 
    Serial.print(String(data));                     // только для отладки нужно
    
    dataString += String(data);                     // добавляем записанный байт в строку(она и есть буфер)
  }                                                 // данных не поступает, строка dataString либо с данными, либо пуста

  if (dataString.startsWith("133;1;1;0;0;")) {      // температура 133 ноды
     data1 = dataString.substring(12).toInt();      // округление до целого
    Serial.print(data1);

    Serial.println(" /Temperatura");

  lpTempUpdate = (millis() - lpTempOldUpdate) / 1000;
  Serial.print(F("Update: ")); Serial.println(lpTempUpdate);
  lpTempOldUpdate = millis();   
  
  }
  
  if (dataString.startsWith("133;2;1;0;1;")) {      // проверяем начало строки на нужное нам совпадение
     data2 = (dataString.substring(12)).toInt();    // вырезаем из строки нужные нам данные, преобразуем данные из String в int и присваиваем их переменной data2
    Serial.print(data2);                            // влажность 133 ноды
    Serial.println(" /Hymidyti");

  }

   if (dataString.startsWith("60;0;1;0;0;")) {      // проверяем начало строки на нужное нам совпадение
     data3 = (dataString.substring(11)).toInt();    // вырезаем из строки нужные нам данные, преобразуем данные из String в int и присваиваем их переменной data3
    Serial.print(data3);                            // температура 133 ноды
    Serial.println(" /Temperatura");
 //   Serial.println(lpContUp);

  lpContUpdate = (millis() - lpContOldUpdate) / 1000;
  Serial.print(F("Update: ")); Serial.println(lpContUpdate);
  lpContOldUpdate = millis();

  } 
  
    if (dataString.startsWith("60;1;1;0;1;")) {     // проверяем начало строки на нужное нам совпадение
     data4 = (dataString.substring(11)).toInt();    // вырезаем из строки нужные нам данные, преобразуем данные из String в int и присваиваем их переменной data4
    Serial.print(data4);                            // влажность 60 ноды
    Serial.println(" /Hymidyti");

  }

    if (dataString.startsWith("60;2;1;0;4;"))  {     // проверяем начало строки на нужное нам совпадение
     data5 = (dataString.substring(11)).toInt();    // вырезаем из строки нужные нам данные, преобразуем данные из String в int и присваиваем их переменной data5
    Serial.print(data5);                            // давление 60 ноды
    Serial.println(" /Pressure");

  }

}

